@@ -201,7 +201,6 @@ impl StockfishActor {
         stdin.write_all(format!("setoption name Use NNUE value {}\n", position.flavor.eval_flavor().is_nnue()).as_bytes()).await?;
         let variant = Variant::from(position.variant);
         if position.flavor == EngineFlavor::MultiVariant {
-            stdin.write_all(b"setoption name Analysis Contempt value Off\n").await?;
             stdin.write_all(format!("setoption name UCI_Variant value {}\n", variant.uci()).as_bytes()).await?;
         }
         stdin.write_all(format!("setoption name MultiPV value {}\n", position.work.multipv()).as_bytes()).await?;
@@ -321,7 +320,7 @@ impl StockfishActor {
                             }
                             "pv" => {
                                 let mut pv = Vec::new();
-                                while let Some(part) = parts.next() {
+                                for part in &mut parts {
                                     pv.push(part.parse().map_err(|_| io::Error::new(io::ErrorKind::InvalidData, "invalid pv"))?);
                                 }
                                 pvs.set(multipv, depth, pv);
